/*
 * fische-3.2
 * Copyright (C) Marcel Ebmer 2009-2010 <marcel@26elf.at>
 *
 * fische-3.2 is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * fische-3.2 is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "fishBMC.h"

#include <cstring>
#include <cstdlib>
#include <unistd.h>
#include <stdexcept>
#include "blurengine.h"
#include "screenbuffer.h"
#include "configuration.h"
#include "vectorfield.h"

#ifdef WIN32
#include <SDL.h>
#else
#include <SDL/SDL.h>
#endif

extern "C" {
#include "traditional.h"
}

using namespace std;

BlurEngine* BlurEngine::ms_instance = 0;

BlurEngine::BlurEngine()
{
	Configuration* config = Configuration::instance();
	mp_ScreenBuffer = ScreenBuffer::instance();
	m_X0 = 0;
	m_Y0 = 0;
	m_Width = config->width();
	m_Height = config->height();

	mp_Buffer = new uint32_t[m_Width * m_Height];
	if (!mp_Buffer)
		throw std::runtime_error ("fishBMC init failed: out of memory");
	m_Cpus = config->processorCores();
	DEBUGOUT ("new blurengine");
}


BlurEngine::~BlurEngine()
{
	delete [] (uint32_t*) mp_Buffer;
	mp_Buffer = NULL;
	DEBUGOUT ("delete blurengine");
}

BlurEngine* BlurEngine::instance()
{
	if (!ms_instance) ms_instance = new BlurEngine();
	return ms_instance;
}

void BlurEngine::release()
{
	delete ms_instance;
	ms_instance = 0;
}

void BlurEngine::blur()
{
	while (!mp_ScreenBuffer->lock())
		usleep (1);

	blurTrad_MT();
	writeBuffer();

	mp_ScreenBuffer->unlock();
}

void BlurEngine::blurTrad_MT()
{
	int n = m_Cpus;
	if (n > 8) n = 8;
	if (n % 2 != 0) n--;
	if (n == 0) n = 1;

	SDL_Thread* t[n];

	blur_param_t p[n];
	for (int i = 0; i < n; i ++) {
		p[i].pixels = mp_ScreenBuffer->pixels();
		p[i].width = m_Width;
		p[i].y0 = m_Y0;
		p[i].x0 = m_X0;
		p[i].pitch = mp_ScreenBuffer->width();
		p[i].vectors = (int8_t*) VectorField::instance()->get();
		p[i].buffer = mp_Buffer;
		p[i].yMin = (i * m_Height) / n;
		p[i].yMax = ( (i + 1) * m_Height) / n;

		t[i] = SDL_CreateThread (traditional_blur, &p[i]);
	}
	for (int i = 0; i < n; i ++) {
		SDL_WaitThread (t[i], NULL);
	}
}

void BlurEngine::emptyBuffer()
{
	memset (mp_Buffer, 0, m_Width * m_Height * 4);
}

void BlurEngine::writeBuffer()
{
	uint32_t* pixels = (uint32_t*) mp_ScreenBuffer->pixels();
	uint32_t* src = (uint32_t*) mp_Buffer;
	int pitch = mp_ScreenBuffer->width();
	for (int y = 0; y < m_Height; y ++) {
		memcpy (pixels + (y + m_Y0) * pitch + m_X0, src + y * m_Width, m_Width * 4);
	}
}
