
/*
 * fische-3.2
 * Copyright (C) Marcel Ebmer 2009-2010 <marcel@26elf.at>
 *
 * fische-3.2 is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * fische-3.2 is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "fishBMC.h"

#include <getopt.h>
#include <unistd.h>
#include <cstdlib>
#include <cstring>
#include <fstream>
#include <cstdio>
#include <stdint.h>
#include "configuration.h"

using namespace std;

Configuration* Configuration::ms_instance = 0;

Configuration::Configuration()
{
	m_IsNervous = false;
	m_Width = 512;
	m_Height = 256;
	m_FrameDivisor = 2;
	m_AspectRatio = 2;
	m_ProcessorCores = cpu_detect();
	DEBUGOUT ("new configuration");
}

Configuration::~Configuration()
{
	DEBUGOUT ("delete configuration");
}

Configuration* Configuration::instance()
{
	if (!ms_instance) ms_instance = new Configuration();
	return ms_instance;
}

void Configuration::release()
{
	delete ms_instance;
	ms_instance = 0;
}

void Configuration::setGeometry (int h)
{
	m_Height = 64;
	while (m_Height < h)
		m_Height *= 2;
	m_Width = m_Height * 2;
}

void Configuration::setNervous (bool n)
{
	m_IsNervous = n;
}

#if defined(__i386__) || defined(__x86_64__) /* OK to use cpuid */
void host_get_cpuid (uint32_t t, uint32_t* eax, uint32_t* ebx, uint32_t* ecx, uint32_t* edx)
{
	asm ("cpuid" : "=a" (*eax), "=b" (*ebx), "=c" (*ecx), "=d" (*edx) : "a" (t), "c" (0), "b" (0), "d" (0));
}
#endif

int Configuration::cpu_detect()
{

#if defined(__i386__) || defined(__x86_64__) /* OK to use cpuid */
	int n = 1;
	uint32_t eax, ebx, ecx, edx;
	host_get_cpuid (0x0, &eax, &ebx, &ecx, &edx);
	char cpuid[13];
	strncpy (cpuid, (char*) &ebx, 4);
	strncpy (cpuid + 4, (char*) &edx, 4);
	strncpy (cpuid + 8, (char*) &ecx, 4);
	cpuid[12] = '\0';

	if (!strcmp (cpuid, "AuthenticAMD")) {
		host_get_cpuid (0x80000000, &eax, &ebx, &ecx, &edx);
		if (eax < 0x80000008) return 1;
		host_get_cpuid (0x80000008, &eax, &ebx, &ecx, &edx);
		n = (ecx & 0xff) + 1;
	} else { /* if it's not AMD, it's most likely Intel */
		host_get_cpuid (0x00000000, &eax, &ebx, &ecx, &edx);
		if (eax < 4) return 1;
		host_get_cpuid (0x4, &eax, &ebx, &ecx, &edx);
		n = (eax >> 26) + 1;
	}
	return n;

#else /* don't have cpuid */
	return 1;
#endif
}

void Configuration::setFrameDivisor (int f)
{
	m_FrameDivisor = f;
}

void Configuration::setAspectRatio (double a)
{
	m_AspectRatio = a;
}
