#ifndef FISHBMC_H

#ifdef WIN32
#include <SDL.h>
#else
#include <SDL/SDL.h>
#endif
#include <GL/gl.h>

#include "configuration.h"

class ScreenBuffer;
class WavePainter;
class BlurEngine;
class Analyst;
class VectorField;
class RingBuffer;

class FishBMC
{
	public:
		FishBMC (struct VIS_PROPS*);
		~FishBMC();
		void render();
		void start();
		void stop();
		void newData (const void*, int);

	private:
		SDL_Thread*		mp_VectorThread;
		SDL_Thread*		mp_BusyThread;
		bool			m_InitBusy;
		bool			m_CancelInit;
		GLuint			m_TextureID;
		double			m_VectorProgress;
		double			m_TextureLeft;
		double			m_TextureRight;
		int				m_FrameCounter;

		Configuration*	mp_config;
		ScreenBuffer* 	mp_screenbuffer;
		WavePainter*	mp_wavepainter;
		Analyst*		mp_analyst;
		VectorField*	mp_vectorfield;
		BlurEngine*		mp_blurengine;
		RingBuffer*		mp_ringbuffer;

		static int 	makevectors (void*);
		void		paint_frame();
		void		try_init_complete();
		void		renderGL();
};

#endif

#ifdef DEBUG
#include <iostream>
using namespace std;
#define DEBUGOUT(A) cout << A << endl
#else
#define DEBUGOUT(A) (void)A
#endif
