
/*
 * fische-3.2
 * Copyright (C) Marcel Ebmer 2009-2010 <marcel@26elf.at>
 *
 * fische-3.2 is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * fische-3.2 is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "fishBMC.h"

#include <cstdlib>
#include <cstring>
#include <stdexcept>
#include <unistd.h>
#ifdef WIN32
#include <SDL.h>
#else
#include <SDL/SDL.h>
#endif
#include "ringbuffer.h"
#include "xbmc/utils/TimeUtils.h"

RingBuffer* RingBuffer::ms_instance = 0;

RingBuffer::RingBuffer (int size)
{
	while (size % 4 != 0) size ++;
	mp_Buffer = new char[size];
	mp_ExportBuffer = new char[size];
	if ( (!mp_ExportBuffer) || (!mp_Buffer))
		throw std::runtime_error ("fishBMC: cannot allocate memory for ringbuffer");
	m_InsertCursor = 0;
	m_ExtractCursor = 0;
	m_IsLocked = false;
	m_Size = size;
	m_Rate = 176400;
	DEBUGOUT ("new ringbuffer");
}

RingBuffer::~RingBuffer()
{
	delete[] mp_Buffer;
	delete[] mp_ExportBuffer;
	DEBUGOUT ("delete blurengine");
}

void RingBuffer::release()
{
	delete ms_instance;
	ms_instance = 0;
}

RingBuffer* RingBuffer::instance (int size)
{
	if (!ms_instance) ms_instance = new RingBuffer (size);
	return ms_instance;
}

void RingBuffer::insert (char* src, int n)
{
	static int last_data = CTimeUtils::GetTimeMS();
	int msecs = CTimeUtils::GetTimeMS() - last_data;
	last_data = CTimeUtils::GetTimeMS();

	if (msecs)
		m_Rate = (n * 1000) / msecs;

	if (n > m_Size)
		n = m_Size;
	while (m_IsLocked) usleep (89);
	m_IsLocked = true;
	int d = m_Size - m_InsertCursor;
	if (d >= n) memcpy (mp_Buffer + m_InsertCursor, src, n);
	else {
		memcpy (mp_Buffer + m_InsertCursor, src, d);
		memcpy (mp_Buffer, src + d, n - d);
	}
	m_InsertCursor += n;
	while (m_InsertCursor >= m_Size) m_InsertCursor -= m_Size;
	m_IsLocked = false;
}

char* RingBuffer::get (double t, int* n)
{
	*n = (int) (t * m_Rate);
	if (*n > m_Size)
		*n = m_Size;
	while ( (*n % 4) != 0)
		(*n) += 1;
	delete[] mp_ExportBuffer;
	mp_ExportBuffer = new char[*n];

	while (m_IsLocked) usleep (97);
	m_IsLocked = true;
	int d = m_Size - m_ExtractCursor;
	if (d >= *n) memcpy (mp_ExportBuffer, mp_Buffer + m_ExtractCursor, *n);
	else {
		memcpy (mp_ExportBuffer, mp_Buffer + m_ExtractCursor, d);
		memcpy (mp_ExportBuffer + d, mp_Buffer, *n - d);
	}
	m_ExtractCursor += *n;
	while (m_ExtractCursor >= m_Size) m_ExtractCursor -= m_Size;
	m_IsLocked = false;
	return mp_ExportBuffer;
}

int RingBuffer::size() const
{
	return m_Size;
}
