
/*
 * fische-3.2
 * Copyright (C) Marcel Ebmer 2009-2010 <marcel@26elf.at>
 *
 * fische-3.2 is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * fische-3.2 is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "fishBMC.h"

#include <iostream>
#include <stdexcept>
#include <cstdlib>
#include <cstring>
#include <stdint.h>
#include "screenbuffer.h"

using namespace std;

ScreenBuffer* ScreenBuffer::ms_instance = 0;

ScreenBuffer::ScreenBuffer (int x, int y)
{
	m_Width = x;
	m_Height = y;

	mp_Pixels = new uint32_t[m_Width * m_Height];
	memset (mp_Pixels, '\0', (m_Width * m_Height * 4));
	m_Locked = false;

	if (!mp_Pixels) {
		cerr << "FishBMC ERROR: could not allocate memory" << endl;
		exit (EXIT_FAILURE);
	}
	DEBUGOUT ("new screenbuffer");
}

ScreenBuffer::~ScreenBuffer()
{
	delete[] (uint32_t*) mp_Pixels;
	DEBUGOUT ("delete screenbuffer");
}

void ScreenBuffer::release()
{
	delete ms_instance;
	ms_instance = 0;
}

ScreenBuffer* ScreenBuffer::instance (int x, int y)
{
	if (!ms_instance) ms_instance = new ScreenBuffer (x, y);
	return ms_instance;
}

int ScreenBuffer::height() const
{
	return m_Height;
}

int ScreenBuffer::width() const
{
	return m_Width;
}

void* ScreenBuffer::pixels()
{
	return mp_Pixels;
}

bool ScreenBuffer::lock()
{
	if (m_Locked) return false;
	m_Locked = true;
	return true;
}

bool ScreenBuffer::unlock()
{
	if (!m_Locked) return false;
	m_Locked = false;
	return true;
}

int ScreenBuffer::pitch() const
{
	return m_Width * 4;
}
